<?php

  namespace Drupal\light_category\Plugin\Block;

  use \Drupal\Core\Block\BlockBase;
  use \Drupal\Core\Block\BlockPluginInterface;
  use \Drupal\Core\Form\FormBuilderInterface;
  use \Drupal\Core\Form\FormStateInterface;
  use \Drupal\light_category\Traits\LightCategoryTrait;
  use \Drupal\views\Views;

  /**
   * Provides a 'Light Category Block'.
   * 
   * @Block(
   * id = "light_category_block",
   * admin_label = @Translation("Light Category Block"),
   * category = @Translation("Views"),
   * )
   */
  class LightCategoryBlock extends BlockBase implements BlockPluginInterface {

  use LightCategoryTrait;

  public static $ONLY_BLOCK = TRUE;

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state )
  {
      $form             = parent::blockForm($form, $form_state);
      $config           = $this->getConfiguration();
      $manager          = \Drupal::entityTypeManager();
 
      $view             = !empty(   $config['view']       )    ?  $config['view']         : [0=>'None'];
      $display          = !empty(   $config['display']    )    ?  $config['display']      : [0=>'None'];

      $arguments        = !empty(   $config['arguments']  )    ?  $config['arguments']    : '';
      $arguments_access = !empty(   $config['display']  )      ?  TRUE                    :  FALSE;

     \Drupal::messenger()->addMessage(  $this->t('You must select a view that contains only a block!') ,'warning');

      $form['light-category-wrapper'] = [
        '#type'       => 'container',
        '#attributes' => [
          'id'          => 'light-category-wrapper',
        ],
      ];
      $form['light-category-wrapper']['light-category-autocomplite'] = [
        '#type'       => 'container',
        '#attributes' => [
          'id'          => 'light-category-autocomplite'
        ],
        '#weight'     => 100,
      ];
      $form['light-category-wrapper']['views'] = [
        '#type'           => 'select',
        '#title'          => $this->t('Select a views'),
        '#options'        => LightCategoryTrait::getSelect( 'view' ),
        '#default_value'  => $view,
        '#required'       => TRUE,
        '#id'             => 'light_category_views',
        '#ajax'           => [
          'callback'        => [$this, 'lightCategoryCallback'],
          'event'           => 'change',
          'method'          => 'replace',
          'wrapper'         => 'light-category-wrapper'
        ]
      ];

      $form['light-category-wrapper']['display'] = [
        '#type'           => 'select',
        '#title'          => $this->t('Select a display'),
        '#options'        => [$display=>$display],
        '#default_value'  => $display,
        '#id'             => 'light_category_display',
        '#required'       => TRUE,
        '#validated'      => TRUE,
        '#ajax'           => [
          'callback'        => [$this, 'lightCategoryCallback'],
          'event'           => 'change',
          'method'          => 'replace',
          'wrapper'         => 'light-category-autocomplite'
        ],
      ];

      $form['light-category-wrapper']['arguments'] = [
        '#type'          => 'entity_autocomplete',
        '#target_type'   => 'taxonomy_term',
        '#title'         => $this->t('View arguments'),
        '#description'   => $this->t('Select the terms you want to set as a contextual filter for the view.'),
        '#default_value' => !empty($arguments) ? $manager->getStorage('taxonomy_term')->load($arguments) : '',
        '#autocreate'    => FALSE,
        '#required'      => FALSE,
        '#tag'           => FALSE,
        '#access'        => TRUE,
        '#prefix'        => '<div id="light-category-autocomplite">',
        '#suffix'        => '</div>'
      ];
      
      return $form;
    }

    /**
     * Ajax callback
     *
     * @param array $form
     * @param FormStateInterface $form_state
     * @return void
     */
    public function lightCategoryCallback(&$form, FormStateInterface &$form_state)
    {

      $values = $form_state->getValues()['settings']['light-category-wrapper'];
      $field_id = $form_state->getTriggeringElement()['#id'];

      switch( $field_id ) {

        case 'light_category_views':
          if( $values['views']  ) {
            $form['settings']['light-category-wrapper']['display']['#options']  = LightCategoryTrait::getSelect( 'display', $values['views'] );
            if( empty($form['settings']['light-category-wrapper']['display']['#options']) ) {
              \Drupal::messenger()->deleteAll();
              \Drupal::messenger()->addMessage($this->t('This view not have a display plugin "Block" '),'error');
            }
            if ( count($form['settings']['light-category-wrapper']['display']['#options']) == 1 ) {
              \Drupal::messenger()->deleteAll();
              $form['settings']['light-category-wrapper']['arguments']['#access'] = TRUE;
            } else {
              $form['settings']['light-category-wrapper']['arguments']['#access'] = FALSE;
            }
            return $form['settings']['light-category-wrapper'];
          } 
        break;

        case 'light_category_display':
          $form['settings']['light-category-wrapper']['arguments']['#access'] = TRUE;
          return $form['settings']['light-category-wrapper']['arguments'];
        break;

        default:
          return $form['settings']['light-category-wrapper'];
        break;
      }

    }

    /**
     * {@inheritdoc}
     */
    public function blockSubmit($form, FormStateInterface $form_state) 
    {
      parent::blockSubmit($form, $form_state);
      $values = $form_state->getValues()['light-category-wrapper'];
      $this->configuration['view']      = $values['views'];
      $this->configuration['display']   = $values['display'];
      $this->configuration['arguments'] = $values['arguments'];
    }

    /**
     * {@inheritdoc}
     */
    public function build()
    {
      \Drupal::messenger()->deleteAll();
      $view       = $this->configuration['view'];
      $display    = $this->configuration['display'];
      $arguments  = $this->configuration['arguments'];
      $view = Views::getView( $view );
      if( !empty( $view ) &&  $view->access($display) ){
            $view->setDisplay($display);
            $view = $view->getDisplay($display);
            $view->view->setArguments( [ $arguments ] );
            $view->preExecute();
            $view->execute();
            $view =  $view->buildRenderable();
            if( $view['#view']->getDisplay()->options['block_hide_empty'] && empty($view['#view']->result) ) {
              return [];
            } else {
              return  $view;
            }
      } else {
        return [
          '#prefix' => '<div class="view-empty">',
          '#markup' => $this->t('Sorry, but it looks like you have deleted the view / display of the view, or the arguments are incorrect.'),
          '#suffix' => '</div>',
        ];
      }
    }
  }